@@ -1,11 +1,14 @@
-/**************************************************************************/
-/*  Klavaro - a flexible touch typing tutor                               */
-/*  Copyright (C) 2005 - 2008  Felipe Castro                              */
-/*                                                                        */
-/*  This program is free software, licensed under the terms of the GNU    */
-/*  General Public License as published by the Free Software Foundation,  */
-/*  which is also included in this package, in a file named "COPYING".    */
-/**************************************************************************/
+/*****************************************************************************/
+/*  Klavaro - a flexible touch typing tutor                                  */
+/*  Copyright (C) 2005, 2006, 2007, 2008 Felipe Castro                       */
+/*  Copyright (C) 2009 Free Software Foundation                              */
+/*                                                                           */
+/*  This program is free software, licensed under the terms of the GNU       */
+/*  General Public License as published by the Free Software Foundation,     */
+/*  either version 3 of the License, or (at your option) any later version.  */
+/*  You should have received a copy of the GNU General Public License        */
+/*  along with this program.  If not, see <http://www.gnu.org/licenses/>.    */
+/*****************************************************************************/
 
 /*
  * Adaptability exercise
@@ -16,18 +19,13 @@
 #include <glib.h>
 #include <gtk/gtk.h>
 
-#include "support.h"
+#include "auxiliar.h"
 #include "main.h"
 #include "keyboard.h"
 #include "tutor.h"
 #include "translation.h"
 #include "adaptability.h"
 
-/**********************************************************************
- * Global variables
- */
-extern GtkWidget *window_tutor_;
-
 /**********************************************************************
  * Writes a random pattern of weird words in the exercise window
  */
@@ -199,18 +197,20 @@ adapt_comment (gdouble accuracy)
 			g_strdup (_
 				  (" You are doing well. But...\n"
 				   " Could you make the accuracy reach 95%?\n"));
-	else if (accuracy < 98)
-		tmp_str = g_strdup (_(" You are almost there,"
-				      " but your accuracy rate is still below 98%.\n"
-				      " Try a few more times,"
-				      " or maybe you're getting upset, so go to other kind of exercise.\n"));
+	else if (accuracy < tutor_goal_accuracy ())
+		tmp_str = g_strdup_printf (_(" You are almost there,"
+				" but your accuracy rate is still below %.0f%%.\n"
+				" Try a few more times,"
+				" or maybe you're getting upset, so go to other kind of exercise.\n"),
+			       	tutor_goal_accuracy ());
 	else
-		tmp_str = g_strdup (_(" Very good!\n"
-				      " You succeeded with an accuracy rate above 98%.\n"
+		tmp_str = g_strdup_printf (_(" Very good!\n"
+				      " You succeeded with an accuracy rate above %.0f%%.\n"
 				      " Now it is time to increase your velocity.\n"
-				      " Go to the 3rd exercise at the main menu.\n"));
+				      " Go to the 3rd exercise at the main menu.\n"),
+			       	tutor_goal_accuracy ());
 
-	wg = lookup_widget (window_tutor_, "text_tutor");
+	wg = get_wg ("text_tutor");
 	buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (wg));
 	gtk_text_buffer_insert_at_cursor (buf, tmp_str, strlen (tmp_str));
 	g_free (tmp_str);
