@@ -1,22 +1,23 @@
-/**************************************************************************/
-/*  Klavaro - a flexible touch typing tutor                               */
-/*  Copyright (C) 2005 - 2008  Felipe Castro                              */
-/*                                                                        */
-/*  This program is free software, licensed under the terms of the GNU    */
-/*  General Public License as published by the Free Software Foundation,  */
-/*  which is also included in this package, in a file named "COPYING".    */
-/**************************************************************************/
+/*****************************************************************************/
+/*  Klavaro - a flexible touch typing tutor                                  */
+/*  Copyright (C) 2005, 2006, 2007, 2008 Felipe Castro                       */
+/*  Copyright (C) 2009 Free Software Foundation                              */
+/*                                                                           */
+/*  This program is free software, licensed under the terms of the GNU       */
+/*  General Public License as published by the Free Software Foundation,     */
+/*  either version 3 of the License, or (at your option) any later version.  */
+/*  You should have received a copy of the GNU General Public License        */
+/*  along with this program.  If not, see <http://www.gnu.org/licenses/>.    */
+/*****************************************************************************/
 
 /*
  * Some auxiliar functions for cursor operations on a GtkTextView widget
  */
 #include "gtk/gtk.h"
 
-#include "support.h"
+#include "auxiliar.h"
 #include "cursor.h"
 
-extern GtkWidget *window_tutor_;
-
 struct
 {
 	gint id;
@@ -54,7 +55,7 @@ cursor_advance (gint n)
 	GtkAdjustment *scroll;
 	GtkTextMark *mark;
 
-	wg = lookup_widget (window_tutor_, "text_tutor");
+	wg = get_wg ("text_tutor");
 	buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (wg));
 
 	gtk_text_buffer_get_iter_at_mark (buf, &new_start, gtk_text_buffer_get_insert (buf));
@@ -85,7 +86,7 @@ cursor_advance (gint n)
 		{
 			gtk_text_view_scroll_mark_onscreen (GTK_TEXT_VIEW (wg),
 							    gtk_text_buffer_get_insert (buf));
-			wg = lookup_widget (window_tutor_, "scrolledwindow_tutor_main");
+			wg = get_wg ("scrolledwindow_tutor_main");
 			scroll = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (wg));
 			gtk_adjustment_set_value (scroll,
 						  gtk_adjustment_get_value (scroll) + 8 * 23 *
@@ -107,7 +108,7 @@ cursor_paint_char (gchar * color_tag_name)
 	GtkTextIter start;
 	GtkTextIter end;
 
-	wg = lookup_widget (window_tutor_, "text_tutor");
+	wg = get_wg ("text_tutor");
 	buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (wg));
 	gtk_text_buffer_get_iter_at_mark (buf, &start, gtk_text_buffer_get_insert (buf));
 	end = start;
@@ -130,7 +131,7 @@ cursor_get_char ()
 	GtkTextIter start;
 	GtkTextIter end;
 
-	wg = lookup_widget (window_tutor_, "text_tutor");
+	wg = get_wg ("text_tutor");
 	buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (wg));
 	gtk_text_buffer_get_iter_at_mark (buf, &start, gtk_text_buffer_get_insert (buf));
 	gtk_text_buffer_get_iter_at_mark (buf, &end, gtk_text_buffer_get_insert (buf));
@@ -165,7 +166,7 @@ cursor_on (gpointer data)
 	if (cursor.blink == FALSE)
 		return FALSE;
 
-	if (window_tutor_ == NULL)
+	if (! GTK_WIDGET_VISIBLE (get_wg ("window_tutor")))
 	{
 		cursor.blink = FALSE;
 		return FALSE;
@@ -190,7 +191,7 @@ cursor_off (gpointer data)
 	if (cursor.blink == FALSE)
 		return FALSE;
 
-	if (window_tutor_ == NULL)
+	if (! GTK_WIDGET_VISIBLE (get_wg ("window_tutor")))
 	{
 		cursor.blink = FALSE;
 		return FALSE;
@@ -214,7 +215,7 @@ cursor_switch_on ()
 	GtkTextIter start;
 	GtkTextIter end;
 
-	wg_text = GTK_TEXT_VIEW (lookup_widget (window_tutor_, "text_tutor"));
+	wg_text = GTK_TEXT_VIEW (get_wg ("text_tutor"));
 	buf = gtk_text_view_get_buffer (wg_text);
 
 	gtk_text_buffer_get_iter_at_mark (buf, &start, gtk_text_buffer_get_insert (buf));
@@ -234,7 +235,7 @@ cursor_switch_off ()
 	GtkTextIter start;
 	GtkTextIter end;
 
-	wg_text = GTK_TEXT_VIEW (lookup_widget (window_tutor_, "text_tutor"));
+	wg_text = GTK_TEXT_VIEW (get_wg ("text_tutor"));
 	buf = gtk_text_view_get_buffer (wg_text);
 
 	gtk_text_buffer_get_iter_at_mark (buf, &start, gtk_text_buffer_get_insert (buf));
