@@ -1,18 +1,21 @@
-/**************************************************************************/
-/*  Klavaro - a flexible touch typing tutor                               */
-/*  Copyright (C) 2005 - 2008  Felipe Castro                              */
-/*                                                                        */
-/*  This program is free software, licensed under the terms of the GNU    */
-/*  General Public License as published by the Free Software Foundation,  */
-/*  which is also included in this package, in a file named "COPYING".    */
-/**************************************************************************/
+/*****************************************************************************/
+/*  Klavaro - a flexible touch typing tutor                                  */
+/*  Copyright (C) 2005, 2006, 2007, 2008 Felipe Castro                       */
+/*  Copyright (C) 2009 Free Software Foundation                              */
+/*                                                                           */
+/*  This program is free software, licensed under the terms of the GNU       */
+/*  General Public License as published by the Free Software Foundation,     */
+/*  either version 3 of the License, or (at your option) any later version.  */
+/*  You should have received a copy of the GNU General Public License        */
+/*  along with this program.  If not, see <http://www.gnu.org/licenses/>.    */
+/*****************************************************************************/
 
 #ifdef G_OS_UNIX
 # define UNIX_OK TRUE
 # define DIRSEP_S "/"
 # define DIRSEP '/'
-# define LESSON_FONT "Monospace 16"
-# define NORMAL_FONT "Serif 14"
+# define LESSON_FONT "Monospace 14"
+# define NORMAL_FONT "Sans 14"
 #endif
 
 #ifdef G_OS_WIN32
@@ -22,7 +25,7 @@
 # define LESSON_FONT "Courier Bold 14"
 # define NORMAL_FONT "Comic Sans MS 12"
 # undef PACKAGE_LOCALE_DIR
-# define PACKAGE_LOCALE_DIR "..\\lib\\locale"
+# define PACKAGE_LOCALE_DIR "..\\share\\locale"
 # undef PACKAGE_DATA_DIR
 # define PACKAGE_DATA_DIR "..\\share"
 #endif
@@ -61,26 +64,7 @@ void main_preferences_set_boolean (gchar * group, gchar * key, gboolean value);
 
 void main_preferences_save (void);
 
-/*
- * Initialization
+/* Initialization / destruction
  */
-void main_initialize_global_variables (void);
-
-void main_set_auto_state (void);
-
-void main_window_init (void);
-
 void main_window_pass_away (void);
 
-/**********************************************************************
- * Extra functions
- */
-/*
-* Search for the user directory and create it if not found
-*/
-void assert_user_dir (void);
-
-/*
- * Compare two strings, so that it applies to other sorting functions.
- */
-gint comp_str (gconstpointer a, gconstpointer b);
