@@ -1,18 +1,26 @@
-/**************************************************************************/
-/*  Klavaro - a flexible touch typing tutor                               */
-/*  Copyright (C) 2005 - 2008  Felipe Castro                              */
-/*                                                                        */
-/*  This program is free software, licensed under the terms of the GNU    */
-/*  General Public License as published by the Free Software Foundation,  */
-/*  which is also included in this package, in a file named "COPYING".    */
-/**************************************************************************/
+/*****************************************************************************/
+/*  Klavaro - a flexible touch typing tutor                                  */
+/*  Copyright (C) 2005, 2006, 2007, 2008 Felipe Castro                       */
+/*  Copyright (C) 2009 Free Software Foundation                              */
+/*                                                                           */
+/*  This program is free software, licensed under the terms of the GNU       */
+/*  General Public License as published by the Free Software Foundation,     */
+/*  either version 3 of the License, or (at your option) any later version.  */
+/*  You should have received a copy of the GNU General Public License        */
+/*  along with this program.  If not, see <http://www.gnu.org/licenses/>.    */
+/*****************************************************************************/
 
 /**************************************************
  * Contest for fluidness performance
  */
-
+#define DOWNHOST "flip.flop.nom.br/klavaro_top10"
+#define CGI_SERVER "flip.flop.nom.br/cgi-bin/klavaro_rangilo"
 #define MIN_CHARS_TO_LOG 500
 
+#define TIMEOUT 10
+#define LOW_SPEED_LIMIT 160
+#define LOW_SPEED_TIME 5
+
 /**************************************************
  * Structures
  */
@@ -22,7 +30,7 @@ typedef struct USERS
 	gchar *name;
 } User;
 
-#define MAX_NAME_LEN 100
+#define MAX_NAME_LEN 255
 typedef struct STATISTICS
 {
 	gchar lang[2];		// Language code
@@ -37,6 +45,10 @@ typedef struct STATISTICS
 	gchar name[MAX_NAME_LEN + 1];
 } Statistics;
 
+void top10_init (void);
+
+void top10_message (gchar * msg);
+
 void top10_init_stats (gboolean locally);
 
 void top10_clean_stat (gint i, gboolean locally);
