@@ -1,11 +1,14 @@
-/**************************************************************************/
-/*  Klavaro - a flexible touch typing tutor                               */
-/*  Copyright (C) 2005 - 2008  Felipe Castro                              */
-/*                                                                        */
-/*  This program is free software, licensed under the terms of the GNU    */
-/*  General Public License as published by the Free Software Foundation,  */
-/*  which is also included in this package, in a file named "COPYING".    */
-/**************************************************************************/
+/*****************************************************************************/
+/*  Klavaro - a flexible touch typing tutor                                  */
+/*  Copyright (C) 2005, 2006, 2007, 2008 Felipe Castro                       */
+/*  Copyright (C) 2009 Free Software Foundation                              */
+/*                                                                           */
+/*  This program is free software, licensed under the terms of the GNU       */
+/*  General Public License as published by the Free Software Foundation,     */
+/*  either version 3 of the License, or (at your option) any later version.  */
+/*  You should have received a copy of the GNU General Public License        */
+/*  along with this program.  If not, see <http://www.gnu.org/licenses/>.    */
+/*****************************************************************************/
 
 /*
  * Shared tutor window tasks
@@ -15,13 +18,15 @@
 #include <time.h>
 #include <stdio.h>
 #include <locale.h>
+#include <errno.h>
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
+#include <libsexy/sexy-icon-entry.h>
 
-#include "support.h"
-#include "interface.h"
 #include "main.h"
+#include "auxiliar.h"
+#include "callbacks.h"
 #include "translation.h"
 #include "keyboard.h"
 #include "cursor.h"
@@ -32,15 +37,6 @@
 #include "top10.h"
 #include "tutor.h"
 
-/*
- * Variables
- */
-extern GtkWidget *window_main_;
-extern GtkWidget *popup_other_;
-extern gboolean callbacks_shield;
-
-GtkWidget *window_tutor_ = NULL;
-
 #define MAX_TOUCH_TICS 8000
 struct
 {
@@ -56,6 +52,8 @@ struct
 	gint correcting;
 } tutor;
 
+extern gchar *OTHER_DEFAULT;
+
 /*******************************************************************************
  * Interface functions
  */
@@ -91,6 +89,87 @@ tutor_init_timers ()
 	tutor.tmr = g_timer_new ();
 }
 
+gdouble
+tutor_goal_accuracy ()
+{
+	switch (tutor.type)
+	{
+		case TT_BASIC: return 95.0;
+		case TT_ADAPT: return 98.0;
+		case TT_VELO: return 97.0;
+		case TT_FLUID: return 97.0;
+	}
+}
+
+gdouble
+tutor_goal_speed ()
+{
+	switch (tutor.type)
+	{
+		case TT_BASIC: return 10.0;
+		case TT_ADAPT: return 10.0;
+		case TT_VELO: return 50.0;
+		case TT_FLUID: return 50.0;
+	}
+}
+
+gdouble
+tutor_goal_fluidity ()
+{
+	switch (tutor.type)
+	{
+		case TT_BASIC:
+		case TT_ADAPT:
+		case TT_VELO: return 0.0;
+		case TT_FLUID: return 75.0;
+	}
+}
+
+/* Sexy Icon Entry
+ */
+static GtkWidget *entry_mesg = NULL;
+
+void
+on_entry_mesg_restart_clicked (SexyIconEntry *entry, gint pos, gint but, gpointer data)
+{
+	if (tutor_get_query () == QUERY_END)
+		return;
+
+	tutor_set_query (QUERY_INTRO);
+	tutor_process_touch (L'\0');
+}
+
+void
+tutor_entry_mesg_init ()
+{
+	GtkWidget *img;
+
+	img = gtk_image_new_from_stock (GTK_STOCK_CLEAR, GTK_ICON_SIZE_MENU);
+	entry_mesg = sexy_icon_entry_new ();
+	sexy_icon_entry_set_icon (SEXY_ICON_ENTRY (entry_mesg), SEXY_ICON_ENTRY_SECONDARY, GTK_IMAGE (img));
+	sexy_icon_entry_set_icon_highlight (SEXY_ICON_ENTRY (entry_mesg), SEXY_ICON_ENTRY_SECONDARY, TRUE);
+
+	gtk_widget_destroy (get_wg ("entry_mesg"));
+	gtk_box_pack_start_defaults (GTK_BOX (get_wg ("hbox_bottom_mesg")), entry_mesg);
+	gtk_box_reorder_child (GTK_BOX (get_wg ("hbox_bottom_mesg")), entry_mesg, 0);
+
+	g_signal_connect (G_OBJECT (entry_mesg), "icon-pressed", (GCallback) on_entry_mesg_restart_clicked, NULL);
+	g_signal_connect (G_OBJECT (entry_mesg), "drag-drop", (GCallback) on_entry_mesg_drag_drop, NULL);
+	g_signal_connect_after (G_OBJECT (entry_mesg), "focus-out-event", (GCallback) on_entry_mesg_focus_out_event, NULL);
+	g_signal_connect_after (G_OBJECT (entry_mesg), "grab-focus", (GCallback) on_entry_mesg_grab_focus, NULL);
+	g_signal_connect_after (G_OBJECT (entry_mesg), "activate", (GCallback) on_entry_mesg_activate, NULL);
+	g_signal_connect_after (G_OBJECT (entry_mesg), "delete-text", (GCallback) on_entry_mesg_delete_text, NULL);
+	g_signal_connect_after (G_OBJECT (entry_mesg), "insert-text", (GCallback) on_entry_mesg_insert_text, NULL);
+
+	gtk_widget_show (entry_mesg);
+}
+
+GtkWidget *
+tutor_entry_mesg ()
+{
+	return (entry_mesg);
+}
+
 /**********************************************************************
  * Initialize the course 
  */
@@ -102,73 +181,71 @@ tutor_init (TutorType tt_type)
 	GtkWidget *wg;
 	GtkTooltipsData *tt;
 
-	if (window_tutor_ == NULL)
-		window_tutor_ = create_window_tutor ();
-
-	gtk_widget_show (window_tutor_);
-	gtk_widget_hide (window_main_);
-	gtk_widget_grab_focus (lookup_widget (window_tutor_, "entry_mesg"));
+	gtk_widget_hide (get_wg ("window_main"));
+	gtk_widget_hide (get_wg ("window_keyboard"));
+	gtk_widget_hide (get_wg ("dialog_info"));
+	gtk_widget_hide (get_wg ("aboutdialog_klavaro"));
+	gtk_widget_show (get_wg ("window_tutor"));
+	//gtk_widget_grab_focus (get_wg ("entry_mesg"));
+	gtk_widget_grab_focus (entry_mesg); // Sexy Icon Entry...
 
 	tutor.type = tt_type;
 	cursor_set_blink (FALSE);
 
-
 	/******************************
 	 * Set the layout for each exercise type
 	 */
-	gtk_widget_hide (lookup_widget (window_tutor_, "button_tutor_top10"));
+	gtk_widget_hide (get_wg ("entry_custom_basic_lesson"));
 	if (tutor.type == TT_BASIC || tutor.type == TT_FLUID)
 	{
-		gtk_widget_show (lookup_widget (window_tutor_, "frame_lesson"));
-		gtk_widget_show (lookup_widget (window_tutor_, "vseparator_tutor"));
+		gtk_widget_show (get_wg ("label_lesson"));
+		gtk_widget_show (get_wg ("spinbutton_lesson"));
+		gtk_widget_show (get_wg ("vseparator_tutor_2"));
 		if (tutor.type == TT_BASIC)
 		{
-			gtk_widget_show (lookup_widget (window_tutor_, "button_revert_lesson"));
-			gtk_widget_show (lookup_widget (window_tutor_, "button_reset_lesson"));
-			gtk_widget_show (lookup_widget (window_tutor_, "button_tutor_show_keyb"));
-			gtk_widget_hide (lookup_widget (window_tutor_, "button_tutor_top10"));
-			gtk_label_set_text (GTK_LABEL
-					    (lookup_widget (window_tutor_, "label_lesson")),
-					    _("Lesson:"));
-			gtk_spin_button_set_range (GTK_SPIN_BUTTON
-						   (lookup_widget
-						    (window_tutor_, "spinbutton_lesson")), 1, 50);
+			gtk_widget_show (get_wg ("togglebutton_edit_basic_lesson"));
+			gtk_widget_hide (get_wg ("button_tutor_top10"));
+			gtk_widget_show (get_wg ("button_tutor_show_keyb"));
+			gtk_label_set_text (GTK_LABEL (get_wg ("label_lesson")), _("Lesson:"));
+			callbacks_shield_set (TRUE);
+			gtk_spin_button_set_range (GTK_SPIN_BUTTON (get_wg ("spinbutton_lesson")), 1, 50);
+			callbacks_shield_set (FALSE);
 		}
 		else
 		{
-			gtk_widget_hide (lookup_widget (window_tutor_, "button_revert_lesson"));
-			gtk_widget_hide (lookup_widget (window_tutor_, "button_reset_lesson"));
-			gtk_widget_hide (lookup_widget (window_tutor_, "button_tutor_show_keyb"));
-			gtk_widget_show (lookup_widget (window_tutor_, "button_tutor_top10"));
-			gtk_label_set_text (GTK_LABEL
-					    (lookup_widget (window_tutor_, "label_lesson")),
-					    _("Paragraphs:"));
-			callbacks_shield = TRUE;
-			gtk_spin_button_set_range (GTK_SPIN_BUTTON
-						   (lookup_widget
-						    (window_tutor_, "spinbutton_lesson")), 1, 10);
-			callbacks_shield = FALSE;
+			gtk_widget_hide (get_wg ("togglebutton_edit_basic_lesson"));
+			gtk_widget_show (get_wg ("button_tutor_top10"));
+			gtk_widget_hide (get_wg ("button_tutor_show_keyb"));
+			gtk_label_set_text (GTK_LABEL (get_wg ("label_lesson")), _("Paragraphs:"));
+			callbacks_shield_set (TRUE);
+			gtk_spin_button_set_range (GTK_SPIN_BUTTON (get_wg ("spinbutton_lesson")), 1, 10);
+			callbacks_shield_set (FALSE);
 		}
 	}
 	else
 	{
-		gtk_widget_hide (lookup_widget (window_tutor_, "frame_lesson"));
-		gtk_widget_hide (lookup_widget (window_tutor_, "vseparator_tutor"));
-		gtk_widget_hide (lookup_widget (window_tutor_, "button_tutor_show_keyb"));
-		gtk_widget_hide (lookup_widget (window_tutor_, "button_tutor_top10"));
-		basic_set_lesson (0);
+		gtk_widget_hide (get_wg ("label_lesson"));
+		gtk_widget_hide (get_wg ("spinbutton_lesson"));
+		gtk_widget_hide (get_wg ("vseparator_tutor_2"));
+		gtk_widget_hide (get_wg ("togglebutton_edit_basic_lesson"));
+		gtk_widget_hide (get_wg ("button_tutor_show_keyb"));
+		gtk_widget_hide (get_wg ("button_tutor_top10"));
 	}
 
-	wg = lookup_widget (window_tutor_, "button_tutor_other");
 	if (tutor.type == TT_BASIC || tutor.type == TT_ADAPT)
-		gtk_widget_hide (wg);
+	{
+		gtk_widget_hide (get_wg ("button_tutor_other"));
+		gtk_widget_hide (get_wg ("vseparator_tutor_1"));
+	}
 	else
-		gtk_widget_show (wg);
+	{
+		gtk_widget_show (get_wg ("button_tutor_other"));
+		gtk_widget_show (get_wg ("vseparator_tutor_1"));
+	}
 
 	/******************************
 	 * Set decoration texts and tips
 	 */
-
 	switch (tutor.type)
 	{
 	case TT_BASIC:
@@ -195,8 +272,8 @@ tutor_init (TutorType tt_type)
 			g_strdup (_("Fluidness exercises: accuracy typing good sense paragraphs."));
 		break;
 	}
-	gtk_window_set_title (GTK_WINDOW (window_tutor_), tmp_title);
-	wg = lookup_widget (window_tutor_, "label_heading");
+	gtk_window_set_title (get_win ("window_tutor"), tmp_title);
+	wg = get_wg ("label_heading");
 	gtk_label_set_text (GTK_LABEL (wg), tmp_name);
 	g_free (tmp_title);
 	g_free (tmp_name);
@@ -204,28 +281,32 @@ tutor_init (TutorType tt_type)
 	/******************************
 	 * Set tooltips of tutor entry (drag and drop)
 	 */
-	wg = lookup_widget (window_tutor_, "entry_mesg");
-	tt = gtk_tooltips_data_get (wg);
 	if (tutor.type == TT_VELO || tutor.type == TT_FLUID)
-		gtk_tooltips_set_tip (tt->tooltips, wg,
-				      _("Drag and drop text here to practice with it."), NULL);
+		gtk_widget_set_tooltip_text (entry_mesg, _("Drag and drop text here to practice with it.")); // Sexy...
 	else
-		gtk_tooltips_set_tip (tt->tooltips, wg, ";-)", NULL);
+		gtk_widget_set_tooltip_text (entry_mesg, _("Press here to restart the exercise. Hotkey: [Ctrl R]")); //Sexy...
 
 	/******************************
 	 * Set specific variables
 	 */
+	tutor.query = QUERY_INTRO;
 	if (tutor.type == TT_BASIC)
 	{
 		basic_init ();
-		return;
+		if (basic_get_lesson () > 1)
+		{
+			tutor_process_touch ('\0');
+			return;
+		}
 	}
 	else if (tutor.type == TT_VELO)
+	{
 		velo_init ();
+	}
 	else if (tutor.type == TT_FLUID)
+	{
 		fluid_init ();
-
-	tutor.query = QUERY_INTRO;
+	}
 	tutor_update ();
 }
 
@@ -247,20 +328,12 @@ tutor_update ()
 		break;
 
 	case QUERY_PROCESS_TOUCHS:
-		tutor_message (_("STARTED! => "));
-		gtk_widget_show (lookup_widget (window_tutor_, "button_tutor_restart"));
+		//gtk_widget_set_sensitive (get_wg ("button_tutor_restart"), TRUE);
 		break;
 
 	case QUERY_END:
-		if (tutor.type == TT_BASIC)
-			tutor_message (_
-				       ("<<==  End of lesson."
-					" Press [Enter] to start another. ==>> "));
-		else
-			tutor_message (_
-				       ("<<==  End of exercise."
-					" Press [Enter] to start another. ==>> "));
-		gtk_widget_hide (lookup_widget (window_tutor_, "button_tutor_restart"));
+		tutor_message (_("End of exercise. Press [Enter] to start another."));
+		//gtk_widget_set_sensitive (get_wg ("button_tutor_restart"), FALSE);
 		break;
 	}
 }
@@ -269,6 +342,8 @@ void
 tutor_update_intro ()
 {
 	gchar *tmp_name;
+	gchar *text;
+	GdkColor color;
 	GtkWidget *wg;
 	GtkLabel *wg_label;
 	GtkTextView *wg_text;
@@ -278,38 +353,42 @@ tutor_update_intro ()
 
 	if (tutor.type == TT_BASIC)
 	{
-		callbacks_shield = TRUE;
+		callbacks_shield_set (TRUE);
 		gtk_spin_button_set_value (GTK_SPIN_BUTTON
-					   (lookup_widget (window_tutor_, "spinbutton_lesson")),
+					   (get_wg ("spinbutton_lesson")),
 					   basic_get_lesson ());
-		callbacks_shield = FALSE;
+		callbacks_shield_set (FALSE);
 
-		wg_label = GTK_LABEL (lookup_widget (window_tutor_, "label_heading"));
+		wg_label = GTK_LABEL (get_wg ("label_heading"));
 		gtk_label_set_text (wg_label, _("Learning the key positions."));
-		tutor_message (_("Press any key to start the lesson. "));
 	}
-	else
-		tutor_message (_("Press any key to start the exercise. "));
 
-	gtk_widget_hide (lookup_widget (window_tutor_, "button_tutor_restart"));
+	tutor_message (_("Press any key to start the exercise. "));
+
+	//gtk_widget_set_sensitive (get_wg ("button_tutor_restart"), FALSE);
 
 	tmp_name = g_strconcat ("_", tutor_get_type_name (), "_intro.txt", NULL);
-	gchar *text = trans_read_text (tmp_name);
-
+	text = trans_read_text (tmp_name);
 	g_free (tmp_name);
-	wg_text = GTK_TEXT_VIEW (lookup_widget (window_tutor_, "text_tutor"));
+
+	wg_text = GTK_TEXT_VIEW (get_wg ("text_tutor"));
 	gtk_text_buffer_set_text (gtk_text_view_get_buffer (wg_text), text, -1);
-	gtk_text_buffer_get_bounds (gtk_text_view_get_buffer (wg_text), &start, &end);
-	gtk_text_buffer_apply_tag_by_name (gtk_text_view_get_buffer (wg_text), "lesson_font", &start, &end);
 	g_free (text);
 
-	callbacks_shield = TRUE;
-	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_tutor_, "entry_mesg")), "");
-	callbacks_shield = FALSE;
+	gtk_text_buffer_get_bounds (gtk_text_view_get_buffer (wg_text), &start, &end);
+	gtk_text_buffer_apply_tag_by_name (gtk_text_view_get_buffer (wg_text), "lesson_font", &start, &end);
+	gtk_text_buffer_apply_tag_by_name (gtk_text_view_get_buffer (wg_text), "text_intro", &start, &end);
+	gdk_color_parse (TUTOR_WHITE, &color);
+	gtk_widget_modify_base (get_wg ("text_tutor"), GTK_STATE_INSENSITIVE, &color);
+	
 
-	wg = lookup_widget (window_tutor_, "scrolledwindow_tutor_main");
+	wg = get_wg ("scrolledwindow_tutor_main");
 	scroll = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (wg));
 	gtk_adjustment_set_value (scroll, 0);
+
+	callbacks_shield_set (TRUE);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (get_wg ("togglebutton_tutor_intro")), TRUE);
+	callbacks_shield_set (FALSE);
 }
 
 void
@@ -317,7 +396,7 @@ tutor_update_start ()
 {
 	gchar *tmp_name;
 	gchar *text;
-	gint level;
+	GdkColor color;
 	GtkWidget *wg;
 	GtkTextBuffer *buf;
 	GtkTextIter start;
@@ -327,45 +406,24 @@ tutor_update_start ()
 	/*
 	 * Delete all the text on tutor window
 	 */
-	wg = lookup_widget (window_tutor_, "text_tutor");
+	wg = get_wg ("text_tutor");
 	buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (wg));
 	gtk_text_buffer_set_text (buf, "", -1);
 
 	if (tutor.type == TT_BASIC)
 	{
-		callbacks_shield = TRUE;
-		gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget
-							    (window_tutor_, "spinbutton_lesson")),
+		callbacks_shield_set (TRUE);
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (get_wg ("spinbutton_lesson")),
 					   basic_get_lesson ());
-		callbacks_shield = FALSE;
-
-		if (main_preferences_exist ("tutor", "basic_lesson"))
-		{
-			level = main_preferences_get_int ("tutor", "basic_lesson");
-			wg = lookup_widget (window_tutor_, "button_revert_lesson");
-			if (level == basic_get_lesson ())
-				gtk_widget_hide (wg);
-			else
-				gtk_widget_show (wg);
-
-			wg = lookup_widget (window_tutor_, "button_reset_lesson");
-			if (level == 1)
-				gtk_widget_hide (wg);
-			else
-				gtk_widget_show (wg);
-		}
-		else
-		{
-			wg = lookup_widget (window_tutor_, "button_revert_lesson");
-			gtk_widget_show (wg);
-		}
+		callbacks_shield_set (FALSE);
 
 		tmp_name = g_ucs4_to_utf8 (basic_get_char_set (), -1, NULL, NULL, NULL);
-		text = g_strdup_printf (_("Keys: %s"), tmp_name);
+		text = g_strdup_printf ("%s %s", _("Keys:"), tmp_name);
 		g_free (tmp_name);
-		wg = lookup_widget (window_tutor_, "label_heading");
+		wg = get_wg ("label_heading");
 		gtk_label_set_text (GTK_LABEL (wg), text);
 		g_free (text);
+
 		basic_draw_lesson ();
 	}
 
@@ -383,6 +441,8 @@ tutor_update_start ()
 		fluid_draw_random_paragraphs ();
 	}
 
+	//gtk_widget_set_sensitive (get_wg ("button_tutor_restart"), FALSE);
+
 	/*
 	 * Apply tutor background color and font to the text
 	 */
@@ -390,16 +450,18 @@ tutor_update_start ()
 	gtk_text_iter_backward_char (&end);
 	gtk_text_buffer_apply_tag_by_name (buf, "char_untouched", &start, &end);
 	gtk_text_buffer_apply_tag_by_name (buf, "lesson_font", &start, &end);
-
-	gtk_widget_hide (lookup_widget (window_tutor_, "button_tutor_restart"));
+	gdk_color_parse (TUTOR_CREAM, &color);
+	gtk_widget_modify_base (get_wg ("text_tutor"), GTK_STATE_INSENSITIVE, &color);
+	
 	tutor_message (_("Start typing when you are ready. "));
-	callbacks_shield = TRUE;
-	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_tutor_, "entry_mesg")), "");
-	callbacks_shield = FALSE;
 
-	wg = lookup_widget (window_tutor_, "scrolledwindow_tutor_main");
+	wg = get_wg ("scrolledwindow_tutor_main");
 	scroll = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (wg));
 	gtk_adjustment_set_value (scroll, 0);
+
+	callbacks_shield_set (TRUE);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (get_wg ("togglebutton_tutor_intro")), FALSE);
+	callbacks_shield_set (FALSE);
 }
 
 
@@ -410,11 +472,12 @@ void
 tutor_process_touch (gunichar user_chr)
 {
 	gboolean go_on;
+	gchar *u8ch;
 	GtkTextView *wg_text;
 	GtkTextBuffer *wg_buffer;
 	GtkTextIter start;
 
-	wg_text = GTK_TEXT_VIEW (lookup_widget (window_tutor_, "text_tutor"));
+	wg_text = GTK_TEXT_VIEW (get_wg ("text_tutor"));
 	wg_buffer = gtk_text_view_get_buffer (wg_text);
 
 	switch (tutor.query)
@@ -441,6 +504,14 @@ tutor_process_touch (gunichar user_chr)
 	case QUERY_START:
 		tutor.query = QUERY_PROCESS_TOUCHS;
 		tutor_update ();
+		callbacks_shield_set (TRUE);
+		u8ch = g_malloc0 (7);
+		if (g_unichar_to_utf8 (user_chr, u8ch) > 0)
+			tutor_message (u8ch);
+		else
+			tutor_message ("");
+		g_free (u8ch);
+		callbacks_shield_set (FALSE);
 
 		if (tutor.type == TT_BASIC)
 			hints_update_from_char (cursor_get_char ());
@@ -698,7 +769,7 @@ tutor_calc_stats ()
 	seconds = ((gulong) tutor.elapsed_time) % 60;
 
 	accuracy = 100 * (1.0 - (gfloat) tutor.n_errors / tutor.n_touchs);
-	touchs_per_second = (tutor.n_touchs - tutor.n_errors) / tutor.elapsed_time;
+	touchs_per_second = (gdouble) (tutor.n_touchs - tutor.n_errors) / tutor.elapsed_time;
 	velocity = 10 * touchs_per_second;
 
 	if (tutor.type == TT_FLUID)
@@ -731,22 +802,26 @@ tutor_calc_stats ()
 		if (average <= 0)
 			average = 1.0e-9;
 		fluidness = 100 * (1 - standard_deviation / average);
-		if (fluidness < 0)
-			fluidness = 0;
+		if (fluidness < 2)
+			fluidness = 2;
 	}
 	else
 		fluidness = 0;
+	stat.score = 0;
 
 	/* Verify if logging is allowed 
 	 */
 	may_log = TRUE;
 	if (tutor.type == TT_FLUID)
-		if (tutor.n_touchs - tutor.n_errors < MIN_CHARS_TO_LOG)
+		if (tutor.n_touchs < MIN_CHARS_TO_LOG)
 		{
-			gdk_beep (); gdk_beep ();
-			contest_ps = g_strdup_printf (_("ps.: logging not performed for this session: "
-				   "the number of typed characters (%i) must be greater than %i."),
-				   tutor.n_touchs - tutor.n_errors, MIN_CHARS_TO_LOG);
+			gdk_beep ();
+			gdk_beep ();
+			contest_ps =
+				g_strdup_printf (_
+						 ("ps.: logging not performed for this session: "
+						  "the number of typed characters (%i) must be greater than %i."),
+						 tutor.n_touchs, MIN_CHARS_TO_LOG);
 			may_log = FALSE;
 		}
 	if (may_log)
@@ -762,7 +837,7 @@ tutor_calc_stats ()
 		 * Logging
 		 */
 		tmp_name =
-			g_strconcat (main_get_user_dir (), "stat_", tutor_get_type_name (), ".txt",
+			g_strconcat (main_get_user_dir (), G_DIR_SEPARATOR_S "stat_", tutor_get_type_name (), ".txt",
 				     NULL);
 		assert_user_dir ();
 		if (!(fh = (FILE *) g_fopen (tmp_name, "r")))
@@ -812,7 +887,7 @@ tutor_calc_stats ()
 		{
 			/* Log the fluidness results of the last session
 			 */
-			tmp_name = g_strconcat (main_get_user_dir (), "deviation_fluid.txt", NULL);
+			tmp_name = g_build_filename (main_get_user_dir (), "deviation_fluid.txt", NULL);
 			if ((fh = (FILE *) g_fopen (tmp_name, "w")))
 			{
 				g_message ("writing further fluidness results at:\n %s", tmp_name);
@@ -833,11 +908,11 @@ tutor_calc_stats ()
 			/* Add results to Top 10
 			 */
 			tmp_name = main_preferences_get_string ("interface", "language");
-			stat.lang[0] = ( (tmp_name[0] == 'C') ? 'e' : tmp_name[0] );
-			stat.lang[1] = ( (tmp_name[0] == 'C') ? 'n' : tmp_name[1] );
+			stat.lang[0] = ((tmp_name[0] == 'C') ? 'e' : tmp_name[0]);
+			stat.lang[1] = ((tmp_name[0] == 'C') ? 'n' : tmp_name[1]);
 			stat.genv = (UNIX_OK ? 'x' : 'w');
 			stat.when = time (NULL);
-			stat.nchars = tutor.n_touchs - tutor.n_errors;
+			stat.nchars = tutor.n_touchs;
 			stat.accur = accuracy;
 			stat.velo = velocity;
 			stat.fluid = fluidness;
@@ -862,6 +937,14 @@ tutor_calc_stats ()
 						g_strdup (_
 							  ("ps.: you have entered the Top 10 list, great!"));
 					top10_write_stats (LOCAL, -1);
+					if (main_preferences_get_boolean ("game", "autopublish"))
+					{
+						top10_show_stats (LOCAL);
+						top10_show_stats (GLOBAL);
+#ifdef G_OS_UNIX
+						top10_global_publish (NULL);
+#endif
+					}
 				}
 			}
 			else
@@ -879,8 +962,7 @@ tutor_calc_stats ()
 			if (!g_file_test (tmp_name, G_FILE_TEST_IS_REGULAR))
 			{
 				fh = (FILE *) g_fopen (tmp_name, "w");
-				fprintf (fh,
-					 "Score\tDate\tTime\tNumber of chars\tLanguage\n");
+				fprintf (fh, "Score\tDate\tTime\tNumber of chars\tLanguage\n");
 			}
 			else
 				fh = (FILE *) g_fopen (tmp_name, "a");
@@ -889,8 +971,9 @@ tutor_calc_stats ()
 				ltime = localtime (&stat.when);
 				fprintf (fh,
 					 "%3.4f\t%i-%2.2i-%2.2i\t%2.2i:%2.2i\t%i\t%s\n",
-					 stat.score, (ltime->tm_year) + 1900, (ltime->tm_mon) + 1, (ltime->tm_mday),
-					 (ltime->tm_hour), (ltime->tm_min), stat.nchars, trans_get_current_language ());
+					 stat.score, (ltime->tm_year) + 1900, (ltime->tm_mon) + 1,
+					 (ltime->tm_mday), (ltime->tm_hour), (ltime->tm_min),
+					 stat.nchars, trans_get_current_language ());
 				fclose (fh);
 			}
 			else
@@ -917,22 +1000,27 @@ tutor_calc_stats ()
 						  "minutes and", minutes),
 				       " %i ", dngettext (PACKAGE, "second",
 							  "seconds", seconds),
-				       "\n", _("Accuracy:"), " %.1f %%\t\t",
-				       _("Goal:"), " %i %%\n\n", _("Comments:"), "\n", NULL);
+				       "\n", _("Error ratio:"), " %i/%i\n", _("Accuracy:"),
+				       " %.1f%%\t\t", _("Goal:"), " %.0f%%\n\n", _("Comments:"),
+				       "\n", NULL);
 	}
 
-	wg = lookup_widget (window_tutor_, "text_tutor");
+	wg = get_wg ("text_tutor");
 	buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (wg));
 
 	switch (tutor.type)
 	{
 	case TT_BASIC:
-		tmp_str2 = g_strdup_printf (tmp_str, minutes, seconds, accuracy, 95);
+		tmp_str2 =
+			g_strdup_printf (tmp_str, minutes, seconds, tutor.n_errors, tutor.n_touchs,
+					 accuracy, tutor_goal_accuracy ());
 		gtk_text_buffer_insert_at_cursor (buf, tmp_str2, strlen (tmp_str2));
 		basic_comment (accuracy);
 		break;
 	case TT_ADAPT:
-		tmp_str2 = g_strdup_printf (tmp_str, minutes, seconds, accuracy, 98);
+		tmp_str2 =
+			g_strdup_printf (tmp_str, minutes, seconds, tutor.n_errors, tutor.n_touchs,
+					 accuracy, tutor_goal_accuracy ());
 		gtk_text_buffer_insert_at_cursor (buf, tmp_str2, strlen (tmp_str2));
 		adapt_comment (accuracy);
 		break;
@@ -943,17 +1031,17 @@ tutor_calc_stats ()
 						  "minutes and", minutes),
 				       " %i ", dngettext (PACKAGE, "second",
 							  "seconds", seconds),
-				       "\n", _("Accuracy:"), " %.1f %%\t\t",
-				       _("Goal:"), " %i %%", "\n",
-				       _("Characters per second:"),
-				       " %.2f\t\t", _("Goal:"), " %i ",
-				       _("(CPS)"), "\n",
-				       _("Words per minute:"), " %.1f\t\t",
-				       _("Goal:"), " %i ", _("(WPM)"), "\n\n", _("Comments:"), "\n",
+				       "\n", _("Error ratio:"), " %i/%i\n", _("Accuracy:"),
+				       " %.1f%%\t\t", _("Goal:"), " %.0f%%", "\n",
+				       _("Characters per second:"), " %.2f\t\t", _("Goal:"), " %.1f ",
+				       _("(CPS)"), "\n", _("Words per minute:"), " %.1f\t\t",
+				       _("Goal:"), " %.0f ", _("(WPM)"), "\n\n", _("Comments:"), "\n",
 				       NULL);
 		tmp_str2 =
-			g_strdup_printf (tmp_str, minutes, seconds, accuracy, 97, velocity / 10, 5,
-					 velocity, 50);
+			g_strdup_printf (tmp_str, minutes, seconds, tutor.n_errors, tutor.n_touchs,
+					 accuracy, tutor_goal_accuracy (),
+					 velocity / 10, tutor_goal_speed () / 10,
+					 velocity, tutor_goal_speed ());
 		gtk_text_buffer_insert_at_cursor (buf, tmp_str2, strlen (tmp_str2));
 		velo_comment (accuracy, velocity);
 		break;
@@ -964,18 +1052,19 @@ tutor_calc_stats ()
 						  "minutes and", minutes),
 				       " %i ", dngettext (PACKAGE, "second",
 							  "seconds", seconds),
-				       "\n", _("Accuracy:"), " %.1f %%\t\t",
-				       _("Goal:"), " %i %%", "\n",
-				       _("Characters per second:"),
-				       " %.2f\t\t", _("Goal:"), " %i ",
-				       _("(CPS)"), "\n",
-				       _("Words per minute:"), " %.1f\t\t",
-				       _("Goal:"), " %i ", _("(WPM)"), "\n",
-				       _("Fluidness:"), " %.1f %%\t\t",
-				       _("Goal:"), " %i %%\n\n", _("Comments:"), "\n", NULL);
+				       "\n", _("Error ratio:"), " %i/%i\n", _("Accuracy:"),
+				       " %.1f%%\t\t", _("Goal:"), " %.0f%%", "\n",
+				       _("Characters per second:"), " %.2f\t\t", _("Goal:"), " %.1f ",
+				       _("(CPS)"), "\n", _("Words per minute:"), " %.1f\t\t",
+				       _("Goal:"), " %.0f ", _("(WPM)"), "\n", _("Fluidness:"),
+				       " %.1f%%\t\t", _("Goal:"), " %.0f%%\n", _("Score"),
+				       ": %f\n\n", _("Comments:"), "\n", NULL);
 		tmp_str2 =
-			g_strdup_printf (tmp_str, minutes, seconds, accuracy, 98, velocity / 10, 5,
-					 velocity, 50, fluidness, 80);
+			g_strdup_printf (tmp_str, minutes, seconds, tutor.n_errors, tutor.n_touchs,
+					 accuracy, tutor_goal_accuracy (),
+					 velocity / 10, tutor_goal_speed () / 10,
+					 velocity, tutor_goal_speed (),
+					 fluidness, tutor_goal_fluidity (), stat.score);
 		gtk_text_buffer_insert_at_cursor (buf, tmp_str2, strlen (tmp_str2));
 		fluid_comment (accuracy, velocity, fluidness);
 
@@ -996,7 +1085,7 @@ tutor_calc_stats ()
 /**********************************************************************
  * Ensure the user is not trying to type with weird texts in the fluidness contest
  */
-#define DECEIVENESS_LIMIT 0.195
+#define DECEIVENESS_LIMIT 0.195 // 0.135
 gboolean
 tutor_char_distribution_approved ()
 {
@@ -1025,7 +1114,7 @@ tutor_char_distribution_approved ()
 	/* Get model text
 	 */
 	tmp_code = main_preferences_get_string ("interface", "language");
-	tmp_name = g_strconcat (main_get_data_path (), tmp_code, ".paragraphs", NULL);
+	tmp_name = g_strconcat (main_get_data_path (), G_DIR_SEPARATOR_S, tmp_code, ".paragraphs", NULL);
 	g_free (tmp_code);
 	if (!g_file_get_contents (tmp_name, &model.text, NULL, NULL))
 	{
@@ -1038,12 +1127,10 @@ tutor_char_distribution_approved ()
 			return FALSE;
 		}
 	}
-	g_message ("model file: %s", tmp_name);
-	g_free (tmp_name);
 
 	/* Get text under examination
 	 */
-	wg = lookup_widget (window_tutor_, "text_tutor");
+	wg = get_wg ("text_tutor");
 	buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (wg));
 	gtk_text_buffer_get_bounds (buf, &start, &end);
 	exam.text = gtk_text_buffer_get_text (buf, &start, &end, FALSE);
@@ -1073,18 +1160,16 @@ tutor_char_distribution_approved ()
 	}
 	deceiveness = sqrtf (deceiveness / 9);
 
+	g_message ("model file: %s", tmp_name);
+	if (deceiveness < DECEIVENESS_LIMIT)
+		g_print ("Deceiveness: %.3f. OK, it is less than %.3f.\n", deceiveness, DECEIVENESS_LIMIT);
+	else
+		g_print ("Deceiveness: %.3f. It should be less than %.3f.\n", deceiveness, DECEIVENESS_LIMIT);
+
+	g_free (tmp_name);
 	g_free (model.text);
 	g_free (exam.text);
-
-
-	if (deceiveness < DECEIVENESS_LIMIT)
-		return TRUE;
-	else
-	{
-		g_print ("Deceiveness: %g. Should be lesser than %g.\n", deceiveness,
-			 DECEIVENESS_LIMIT);
-		return FALSE;
-	}
+	return (deceiveness < DECEIVENESS_LIMIT);
 }
 
 /**********************************************************************
@@ -1201,7 +1286,7 @@ tutor_draw_paragraph (gchar * utf8_text)
 	else
 		g_error ("draw_paragraph () ==> string error");
 
-	wg = lookup_widget (window_tutor_, "text_tutor");
+	wg = get_wg ("text_tutor");
 	buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (wg));
 
 	tmp2 = g_strconcat (tmp1, keyb_get_utf8_paragraph_symbol (), "\n", NULL);
@@ -1215,16 +1300,24 @@ tutor_draw_paragraph (gchar * utf8_text)
 void
 tutor_message (gchar * mesg)
 {
+	gint pos = 0;
 	GtkWidget *wg;
 
 	if (mesg == NULL)
 	{
-		g_message ("tutor_message() --> can't show NULL message!");
+		g_message ("tutor_message() --> not showing NULL message!");
 		return;
 	}
 
-	wg = lookup_widget (window_tutor_, "label_tutor_mesg");
-	gtk_label_set_text (GTK_LABEL (wg), mesg);
+	if (entry_mesg == NULL) return; // Sexy Icon Entry...
+	wg = entry_mesg; // Sexy...
+	//wg = get_wg ("entry_mesg");
+
+	callbacks_shield_set (TRUE);
+	gtk_editable_delete_text (GTK_EDITABLE (wg), 0, -1);
+	gtk_editable_insert_text (GTK_EDITABLE (wg), g_strdup (mesg), strlen (mesg), &pos);
+	gtk_editable_set_position (GTK_EDITABLE (wg), -1);
+	callbacks_shield_set (FALSE);
 }
 
 /**********************************************************************
@@ -1235,7 +1328,7 @@ tutor_beep ()
 {
 	GtkWidget *wg;
 
-	wg = lookup_widget (window_tutor_, "checkbutton_beep");
+	wg = get_wg ("togglebutton_tutor_beep");
 	if ((GTK_TOGGLE_BUTTON (wg))->active)
 		gdk_beep ();
 }
@@ -1250,6 +1343,14 @@ tutor_load_list_other (gchar * file_name_end, GtkListStore * list)
 	gchar *dentry;
 	GDir *dir;
 	GtkTreeIter iter;
+	static gchar *defstr = NULL;
+
+	if (defstr == NULL)
+		defstr = g_strdup (OTHER_DEFAULT);
+
+	gtk_list_store_clear (list);
+	gtk_list_store_append (list, &iter);
+	gtk_list_store_set (list, &iter, 0, defstr, -1);
 
 	assert_user_dir ();
 	dir = g_dir_open (main_get_user_dir (), 0, NULL);
@@ -1265,7 +1366,7 @@ tutor_load_list_other (gchar * file_name_end, GtkListStore * list)
 			g_free (dentry);
 			continue;
 		}
-		if (strcmp (file_name_end, tmp_str))
+		if (! g_str_equal (file_name_end, tmp_str))
 		{
 			g_free (dentry);
 			continue;
@@ -1278,6 +1379,56 @@ tutor_load_list_other (gchar * file_name_end, GtkListStore * list)
 	}
 	g_dir_close (dir);
 
-	gtk_list_store_append (list, &iter);
-	gtk_list_store_set (list, &iter, 0, _("--> Default"), -1);
+	gtk_widget_set_sensitive (get_wg ("button_other_remove"), FALSE);
+	gtk_widget_set_sensitive (get_wg ("label_other_rename"), FALSE);
+	gtk_widget_set_sensitive (get_wg ("entry_other_rename"), FALSE);
+	gtk_widget_set_sensitive (get_wg ("button_other_apply"), FALSE);
+}
+
+void
+tutor_other_rename (const gchar *new_tx, const gchar *old_tx)
+{
+	if (! g_str_equal (new_tx, old_tx) && 
+	    ! g_str_equal (new_tx, OTHER_DEFAULT) &&
+	    ! g_str_equal (new_tx, "") &&
+	    g_strrstr (old_tx, "*") == NULL )
+	{
+		gchar *old_name;
+		gchar *new_name;
+		gchar *old_file;
+		gchar *new_file;
+
+		if (tutor.type == TT_VELO)
+		{
+			old_name = g_strconcat (old_tx, ".words", NULL);
+			new_name = g_strconcat (new_tx, ".words", NULL);
+		}
+		else
+		{
+			old_name = g_strconcat (old_tx, ".paragraphs", NULL);
+			new_name = g_strconcat (new_tx, ".paragraphs", NULL);
+		}
+		old_file = g_build_filename (main_get_user_dir (), old_name);
+		new_file = g_build_filename (main_get_user_dir (), new_name);
+
+		if (g_file_test (new_file, G_FILE_TEST_IS_REGULAR))
+		{
+			g_message ("File already exists, not renaming.\n\t%s\n", new_file);
+			gdk_beep ();
+		}
+		else
+		{
+			g_printf ("Renaming from:\n\t%s\nTo:\n\t%s\n", old_file, new_file);
+			if (g_rename (old_file, new_file))
+			{
+				g_printf ("Fail: %s\n", strerror (errno));
+			}
+			else
+				g_printf ("Success!\n");
+		}
+		g_free (old_name);
+		g_free (new_name);
+		g_free (old_file);
+		g_free (new_file);
+	}
 }
