@@ -1,11 +1,14 @@
-/**************************************************************************/
-/*  Klavaro - a flexible touch typing tutor                               */
-/*  Copyright (C) 2005 - 2008  Felipe Castro                              */
-/*                                                                        */
-/*  This program is free software, licensed under the terms of the GNU    */
-/*  General Public License as published by the Free Software Foundation,  */
-/*  which is also included in this package, in a file named "COPYING".    */
-/**************************************************************************/
+/*****************************************************************************/
+/*  Klavaro - a flexible touch typing tutor                                  */
+/*  Copyright (C) 2005, 2006, 2007, 2008 Felipe Castro                       */
+/*  Copyright (C) 2009 Free Software Foundation                              */
+/*                                                                           */
+/*  This program is free software, licensed under the terms of the GNU       */
+/*  General Public License as published by the Free Software Foundation,     */
+/*  either version 3 of the License, or (at your option) any later version.  */
+/*  You should have received a copy of the GNU General Public License        */
+/*  along with this program.  If not, see <http://www.gnu.org/licenses/>.    */
+/*****************************************************************************/
 
 /*
  * Velocity exercise
@@ -17,18 +20,13 @@
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
 
-#include "support.h"
+#include "auxiliar.h"
 #include "main.h"
 #include "translation.h"
 #include "keyboard.h"
 #include "tutor.h"
 #include "velocity.h"
 
-/*
- * Global variables
- */
-extern GtkWidget *window_tutor_;
-
 GList *word_list;
 
 struct
@@ -38,6 +36,8 @@ struct
 	gchar *name;
 } dict;
 
+extern gchar *OTHER_DEFAULT;
+
 /*******************************************************************************
  * Interface functions
  */
@@ -85,17 +85,17 @@ velo_init ()
 void
 velo_init_dict (gchar * list_name)
 {
-	static gchar *word_buf = NULL;	/* no doubt: this is initialized only at the first call */
+	static gchar *word_buf = NULL;
 	gchar *word;
 	gchar *tmp_buf;
 	gchar *tmp_name;
 	gchar *tmp_code;
 	gchar *dict_name;
 
-	if (list_name && !g_str_equal (list_name, _("--> Default")) )
+	if (list_name && !g_str_equal (list_name, OTHER_DEFAULT) )
 	{
 		main_preferences_set_string ("tutor", "word_list", list_name);
-		tmp_name = g_strconcat (main_get_user_dir (), list_name, ".words", NULL);
+		tmp_name = g_strconcat (main_get_user_dir (), G_DIR_SEPARATOR_S, list_name, ".words", NULL);
 		g_message ("loading dictionary: %s.words", list_name);
 		dict_name = g_strdup (list_name);
 	}
@@ -103,17 +103,18 @@ velo_init_dict (gchar * list_name)
 	{
 		main_preferences_remove ("tutor", "word_list");
 		tmp_code = main_preferences_get_string ("interface", "language");
-		tmp_name = g_strconcat (main_get_data_path (), tmp_code, ".words", NULL);
+		tmp_name = g_strconcat (main_get_data_path (), G_DIR_SEPARATOR_S, tmp_code, ".words", NULL);
 		g_message ("loading dictionary: %s.words", tmp_code);
 		g_free (tmp_code);
 		dict_name = g_strdup ("Default");
 	}
 
-	if (!g_file_test (tmp_name, G_FILE_TEST_IS_REGULAR) && g_str_equal (dict_name, "Default") )
+	if (!g_file_test (tmp_name, G_FILE_TEST_IS_REGULAR))
 	{
 		g_free (tmp_name);
 		tmp_name = trans_lang_get_similar_file_name (".words");
 		g_message ("not found, loading from file:\n %s", tmp_name);
+		gdk_beep ();
 	}
 
 	if (g_file_get_contents (tmp_name, &tmp_buf, NULL, NULL))
@@ -161,7 +162,6 @@ void
 velo_draw_random_words ()
 {
 	gint i, j;
-	gchar *hlp;
 	gchar *word;
 	struct PARAGRAPH
 	{
@@ -306,8 +306,8 @@ velo_text_write_to_file (gchar * text_raw, gboolean overwrite)
 	gchar *text_filtered;
 	FILE *fh_destiny;
 
-	dictio_name = g_strdup (_("Pasted_or_dropped"));
-	dict_path = g_strconcat (main_get_user_dir (), dictio_name, ".words", NULL);
+	dictio_name = g_strdup_printf ("(%s)", _("Pasted_or_dropped"));
+	dict_path = g_strconcat (main_get_user_dir (), G_DIR_SEPARATOR_S, dictio_name, ".words", NULL);
 	assert_user_dir ();
 	if (!(fh_destiny = (FILE *) g_fopen (dict_path, "w")))
 	{
@@ -368,7 +368,7 @@ velo_create_dict (gchar * file_name, gboolean overwrite)
 	}
 
 	dictio_name = g_path_get_basename (file_name);
-	dict_path = g_strconcat (main_get_user_dir (), dictio_name, ".words", NULL);
+	dict_path = g_strconcat (main_get_user_dir (), G_DIR_SEPARATOR_S, dictio_name, ".words", NULL);
 	assert_user_dir ();
 	if (!(fh_destiny = (FILE *) g_fopen (dict_path, "w")))
 	{
@@ -414,38 +414,37 @@ velo_comment (gdouble accuracy, gdouble velocity)
 	/*
 	 * Comments
 	 */
-	if (accuracy < 97)
-		tmp_str = g_strdup (_(" Your accuracy rate is not good enough.\n"
-				      " I only can rely on you if it is greater than 97%.\n"
-				      " Please calm down and try it again.\n"));
+	if (accuracy < tutor_goal_accuracy ())
+		tmp_str = g_strdup_printf (_(" Your accuracy rate is not good enough.\n"
+				      " I only can rely on you if it is greater than %.0f%%.\n"
+				      " Please calm down and try it again.\n"), tutor_goal_accuracy ());
 	else if (velocity < 10)
 		tmp_str =
 			g_strdup (_(" You are just beginning.\n"
 				" Be patient, try it again every day, rest and don't worry so much:\n"
 				" persistence and practice will improve your velocity.\n"));
 	else if (velocity < 20)
-		tmp_str =
-			g_strdup (_
-				  (" Still away from the highway. You can make better...\n Try at least 20 WPM.\n"));
+		tmp_str = g_strdup (_(" Still away from the highway. You can make better...\n"
+					" Try at least 20 WPM.\n"));
 	else if (velocity < 30)
 		tmp_str = g_strdup (_(" You are doing well, but need to go faster.\n"
 				      " And don't forget the accuracy. Try to get 30 WPM.\n"));
 	else if (velocity < 40)
 		tmp_str =
-			g_strdup (_
-				  (" Fine. Now you need to start running.\n Can you reach 40 WPM?\n"));
-	else if (velocity < 50)
-		tmp_str =
-			g_strdup (_
-				  (" Very good. You are almost there.\n Can you finally reach 50 WPM?\n"));
+			g_strdup (_(" Fine. Now you need to start running.\n"
+						" Can you reach 40 WPM?\n"));
+	else if (velocity < tutor_goal_speed ())
+		tmp_str = g_strdup_printf (_(" Very good. You are almost there.\n "
+					"Can you finally reach %.0f WPM?\n"),
+					tutor_goal_speed ());
 	else if (velocity < 60)
 		tmp_str =
-			g_strdup (_
-				  (" Excellent. For this course, that is enough.\n Try now the fluidness exercises, OK?\n"));
+			g_strdup (_(" Excellent. For this course, that is enough.\n"
+						" Try now the fluidness exercises, OK?\n"));
 	else if (velocity < 70)
 		tmp_str =
-			g_strdup (_
-				  (" Fast! Are you training for a competition?\n So, try to get 70 WPM!\n"));
+			g_strdup (_(" Fast! Are you training for a competition?\n"
+						" So, try to get 70 WPM!\n"));
 	else if (velocity < 80)
 		tmp_str = g_strdup (_(" Top of \"qwerty\"."
 				      " Now it's time to change to the Dvorak mode.\n"
@@ -458,7 +457,7 @@ velo_comment (gdouble accuracy, gdouble velocity)
 				  (" Dvorak master!\n"
 				   " I have no words to express my admiration!\n"));
 
-	wg = lookup_widget (window_tutor_, "text_tutor");
+	wg = get_wg ("text_tutor");
 	buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (wg));
 	gtk_text_buffer_insert_at_cursor (buf, tmp_str, strlen (tmp_str));
 	g_free (tmp_str);
